import re, bson, random, json, hashlib, traceback, uuid, time, jwt
from datetime import datetime, date
import locale

from flask import request

json = json
bson = bson
traceback = traceback
hashlib = hashlib


def xlwt_write_row(ws, row_index, row_values):
    for i, v in enumerate(row_values):
        ws.write(row_index, i, v)


def to_local_time(input_time: datetime):
    return input_time.strftime('%H:%M:%S %d/%m/%Y')


def vn_currency(val):
    try:
        return locale.currency(str(val), grouping=True, symbol=False).split('.')[0].replace(',', '.')
    except Exception as e:

        return val


def xlrd_read_row(ws, row_index, length=2):
    result = []
    for i in range(0, length):
        try:
            val = ws.cell_value(row_index, i)
        except  Exception as e:
            val = ''
        result.append(val)
    return result


def dict_to_json(input_object: dict):
    result = {}
    for key in input_object.keys():
        converted = False
        if isinstance(input_object.get(key), mongoengine.Document):
            try:
                result[key] = input_object.get(key).json_smallest()
            except Exception as e:
                result[key] = input_object.get(key).to_json()
            result[key] = dict_to_json(result[key])
            converted = True
        if isinstance(input_object.get(key), bson.ObjectId):
            result[key] = str(input_object.get(key))
            converted = True
        if isinstance(input_object.get(key), datetime):
            result[key] = timestamp(input_object.get(key))
            result[f'{key}_text'] = to_utc(input_object.get(key))
            converted = True
        if isinstance(input_object.get(key), list) and list(input_object.get(key)).__len__().__gt__(0) and isinstance(
                input_object.get(key)[0], bson.ObjectId):
            result[key] = list(map(lambda e: str(e), input_object.get(key)))
            converted = True
        if isinstance(input_object.get(key), dict):
            result[key] = dict_to_json(input_object.get(key))
            converted = True
        if not converted:
            result[key] = input_object.get(key)
    return result


def to_utc(time):
    return datetime.strftime(time, "%Y-%m-%d %H:%M:%S")


def client_ip_address():
    return request.environ.get('HTTP_X_REAL_IP', request.remote_addr)


def logger(*args):
    print(datetime.now(), '-', args, flush=True, )


ZERO_TIME = datetime.fromtimestamp(0)


def exception_log(**kwargs):
    print(traceback.format_exc(), kwargs, flush=True)


def is_bson_object_id(string: str):
    return bson.ObjectId.is_valid(string)


def to_vietnam_phone_number(input: str):
    return f"84{re.findall('[0-9]{9}$', input)[0]}"


def timestamp(input_date):
    try:
        if isinstance(input_date, date) and not isinstance(input_date, datetime):
            w_date = input_date
            input_date = datetime(w_date.year, w_date.month, w_date.day, 7, 0, 0, 0)
        return int(time.mktime(input_date.timetuple()))
    except Exception as e:
        return 0


def default_if_none(input_value=None, default=None, result_type=None):
    if result_type is None:
        result_type = type(default)
    return default if is_none(input_value) else result_type(input_value)


din = default_if_none


def to_timestamp(dt):
    return timestamp(dt)


def is_object_id(input: str):
    return bson.ObjectId.is_valid(input)


def is_month(input):
    return isinstance(input, int) and 1 <= input <= 12


def is_year(input):
    return isinstance(input, int)


def mongo_iso_format_text():
    return '%Y-%m-%dT%H:%M:%S.%L'


def is_phone(text: str):
    try:
        return not is_none(re.match(r'^[0|84][2-9][0-9]{8,9}$', text))
    except Exception as e:
        return False


def hash_password(password):
    return hashlib.md5(str(password).encode('utf-8')).hexdigest()


def jwt_encode(input_data: dict):
    return jwt.encode(input_data, secret_key, algorithm='HS256').decode('utf-8')


def jwt_decode(token_string: str, key=None):
    if is_none(key):
        key = secret_key
    return jwt.decode(token_string, key, algorithms=['HS256'])


def is_email(text: str):
    return not is_none(re.match(r'^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,4})+$', text))


def is_username(text: str):
    result = re.match('^[a-z][a-z0-9\.@_]{5,35}$', text)
    return not is_none(result)


def is_display_name(text: str):
    x_text = text.strip()
    return x_text.__contains__(' ') and x_text.__len__().__gt__(2)


def is_valid_uuid(val):
    try:
        uuid.UUID(str(val))
        return True
    except ValueError:
        return False


def is_none(object) -> bool:
    return object is None


def array_object_ids(array):
    result = []
    for e in array:
        if isinstance(e, dict):
            result.append(e.get('_ref').id)
        else:
            result.append(e)
    return result


def time_now():
    return datetime.now()


def datetime_from_timestamp(time_stamp: int) -> datetime:
    return datetime.fromtimestamp(time_stamp)


def ObjectID(id):
    try:
        return bson.ObjectId(id)
    except Exception as error:
        return None


def is_bool(input):
    return str(input).lower() in ['true', 'false', '1', '0']


def bool(input):
    return str(input).lower().__eq__('true') or str(input).lower().__eq__('1')


def ts_to_time(input: int) -> datetime:
    try:
        return datetime.fromtimestamp(input)
    except Exception as error:
        return ZERO_TIME


def to_sod(date_time: datetime) -> datetime:
    return date_time.replace(hour=0, minute=0, second=0, microsecond=0)


def to_start_of_day(date_time: datetime) -> datetime:
    return date_time.replace(hour=0, minute=0, second=0, microsecond=0)


def to_end_of_day(date_time: datetime) -> datetime:
    return date_time.replace(hour=23, minute=59, second=59, microsecond=999999)


def to_eod(date_time: datetime) -> datetime:
    return date_time.replace(hour=23, minute=59, second=59, microsecond=999999)


def password_generate(length):
    return ''.join(random.sample('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789', length))


def md5(input_str: str):
    return hashlib.md5(input_str.encode()).hexdigest()
