from flask import request
from datetime import datetime
import re, bson


class RequestParam:
    def exists(self):
        if (not (request.json is None) and self.param_name in request.json) or (
                not (request.form is None) and self.param_name in request.form):
            return True
        else:
            return False

    def __init__(self, param_name):
        self.param_name = param_name

    def text(self):
        if not (request.json is None) and self.param_name in request.json:
            return request.json.get(self.param_name)
        if not (request.form is None) and self.param_name in request.form:
            return request.form.get(self.param_name)
            # if (not (request.data is None)) and property_x in request.data:
            #     return request.data.get(property_x)
        return None

    def context(self):
        return self.text()

    def str(self):
        if not (request.json is None) and self.param_name in request.json:
            return str(request.json.get(self.param_name))
        if not (request.form is None) and self.param_name in request.form:
            return str(request.form.get(self.param_name))
        return ''

    def is_bool(self):
        return str(self.text()).lower() in ['true', 'false', '1', '0']

    def bool(self):
        return str(self.text()).lower().__eq__('true') or str(self.text()).lower().__eq__('1')

    def is_number(self):
        return self.str().isnumeric()

    def number(self):
        return int(self.text())

    def timestamp_as_time(self) -> datetime:
        try:
            return datetime.fromtimestamp(self.number())
        except Exception as error:
            return None

    def is_phone(self):
        return re.match('^[0|84][0-9]{9}$', self.text())

    def is_email(self):
        return re.match(r'^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$', self.text())

    def is_username(self):
        return re.match('^[a-z][a-z0-9.@_]{5,25}$', self.text())

    def is_display_name(self):
        x_text = self.text().strip()
        return x_text.__contains__(' ') and x_text.__len__().__gt__(2)

    def is_none(self):
        return self.text() is None

    def ObjectID(self):
        try:
            return bson.ObjectId(self.text())
        except Exception as error:
            return None


class RequestQuery:
    def exists(self):
        return self.query_name in request.args and len(request.args.get(self.query_name)) > 0

    def __init__(self, query_name):
        self.query_name = query_name

    def text(self) -> str:
        return request.args.get(self.query_name)

    def context(self):
        return self.text()

    def is_bool(self):
        return str(self.text()).lower() in ['true', 'false', '1', '0']

    def bool(self):
        return str(self.text()).lower().__eq__('true') or str(self.text()).lower().__eq__('1')

    def is_number(self):
        return self.text().isnumeric()

    def number(self):
        return int(self.text())

    def timestamp_as_time(self) -> datetime:
        try:
            return datetime.fromtimestamp(self.number())
        except Exception as error:
            return None

    def is_phone(self):
        return re.match('^[0|84][0-9]{9}$', self.text())

    def is_email(self):
        return re.match(r'^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$', self.text())

    def is_username(self):
        return re.match('^[a-z][a-z0-9.@_]{5,25}$', self.text())

    def is_display_name(self):
        x_text = self.text().strip()
        return x_text.__contains__(' ') and x_text.__len__().__gt__(2)

    def is_none(self):
        return self.text() is None

    def ObjectID(self):
        try:
            return bson.ObjectId(self.text())
        except Exception as error:
            return None


def has_json(property_name, target=None):
    return property_name in request.json


class Method:

    def __init__(self):
        self.POST = 'POST'
        self.GET = 'GET'
        self.PUT = 'PUT'
        self.DELETE = 'DELETE'


class UserRole:
    def __init__(self):
        self.ADMIN = 'admin'
        self.STAFF = 'normal'
        self.AFFILIATE = 'affiliate'


class RequestParse:

    def __init__(self):
        self.stack = []
        self.METHOD = Method()
        self.ROLE = UserRole()

    def param(self, param_name) -> RequestParam:
        return RequestParam(param_name=param_name)

    def query(self, query_name) -> RequestQuery:
        return RequestQuery(query_name=query_name)

    def has_params(self, properties: list, target=None) -> bool:
        result = True
        self.stack = []
        if target is None:
            for property_x in properties:
                if not has_json(property_x):
                    result = False
                    self.stack.append(property_x)
        else:
            for property_x in properties:
                if not has_json(property_x, target):
                    result = False
                    self.stack.append(property_x)
        if self.stack.__len__().__gt__(0):
            print('Missing params', self.stack)
        return result

    def PV(self, properties: object = []) -> object:
        result = []
        if str(request.content_type).__contains__('application/json'):
            source = request.json
        else:
            source = request.form
        for attr in properties:
            result.append(source.get(attr))
        return result

    def QV(self, properties: object = []) -> object:
        result = []
        source = request.args
        if source is None:
            source = {}
        for attr in properties:
            result.append(source.get(attr))
        return result

    def stack_param(self):
        return ', '.join(self.stack)

    def has_querys(self, args: list) -> bool:
        self.stack = []
        for arg in args:
            if not (arg in request.args):
                return False
        return True

    def match(self, pattern, string):
        return re.match(pattern, string)


_ = RequestParse()
